'''
Created on May 11, 2012

@author: shane
'''
import os

os.environ['DJANGO_SETTINGS_MODULE'] = 'pykeg.settings'

from google.appengine.ext.webapp import util
import django.core.handlers.wsgi

app = django.core.handlers.wsgi.WSGIHandler()

def main():
  util.run_wsgi_app(app)

if __name__ == '__main__':
  main()



from imagekit.specs import ImageSpec
from imagekit import processors

class ResizeThumb(processors.Resize):
    width = 128
    height = 128
    crop = True

class ResizeDisplay(processors.Resize):
    width = 320

class EnhanceThumb(processors.Adjustment):
    contrast = 1.2
    sharpness = 1.1

class Thumbnail(ImageSpec):
    access_as = 'thumbnail'
    pre_cache = True
    processors = [ResizeThumb, EnhanceThumb]

class Display(ImageSpec):
    increment_count = False
    processors = [ResizeDisplay]





from django.conf.urls.defaults import *

urlpatterns = patterns('pykeg.web.contrib.facebook.views',
    url(r'update-perms/$', 'update_perms', name='fb-update-perms'),
    url(r'status-update/$', 'status_update', name='fb-status-update'),
)



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Wrapper module for database implementation."""

from pykeg.core import kb_common

class BackendError(Exception):
  """Base backend error exception."""

class NoTokenError(BackendError):
  """Token given is unknown."""

class Backend:
  """Abstract base Kegbot backend class.

  This class defines the interface that pykeg uses to talk to the kegbot
  backend.
  """

  def CreateNewUser(self, username, gender=kb_common.DEFAULT_NEW_USER_GENDER,
      weight=kb_common.DEFAULT_NEW_USER_WEIGHT):
    """Creates a new User instance.

    Args
      username: the unique username for the new User
      gender: the gender to assign the new user
      weight: the weight to assign the new user
    Returns
      the new User instance
    """
    raise NotImplementedError

  def GetAllTaps(self):
    """Returns all currently enabled taps."""
    raise NotImplementedError

  def RecordDrink(self, tap_name, ticks, volume_ml=None, username=None,
      pour_time=None, duration=0, auth_token=None, spilled=False,
      shout=''):
    """Records a new drink with the given parameters."""
    raise NotImplementedError

  def CancelDrink(self, seqn, spilled=False):
    """Cancels the given drink.

    If `spilled` is False, the drink will be canceled as if it never occurred.
    Otherwise, the drink will be canceled, and the volume associated with it
    will be transfered to its keg's spilled total.
    """
    raise NotImplementedError

  def LogSensorReading(self, sensor_name, temperature, when=None):
    """Records a new sensor reading."""
    raise NotImplementedError

  def GetAuthToken(self, auth_device, token_value):
    """Returns an AuthenticationToken instance."""
    raise NotImplementedError




# Copyright 2011 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import datetime

from kegbot.util import units

from pykeg.core import models

def to_pints(volume):
  return float(units.Quantity(volume).InPints())

class ChartError(Exception):
  """Base chart exception."""

def TemperatureSensorChart(sensor):
  """ Shows a simple line plot of a specific temperature sensor.

  Syntax:
    {% chart sensor <sensorname> width height %}
  Args:
    sensorname - the nice_name of a ThermoSensor
  """
  if not isinstance(sensor, models.ThermoSensor):
    raise ChartError('Bad sensor given')

  hours = 6
  now = datetime.datetime.now()
  start = now - (datetime.timedelta(hours=hours))
  start = start - (datetime.timedelta(seconds=start.second))

  points = sensor.thermolog_set.filter(time__gte=start).order_by('time')

  curr = start
  temps = []
  have_temps = False
  for point in points:
    while curr <= point.time:
      curr += datetime.timedelta(minutes=1)
      if curr < point.time:
        temps.append(None)
      else:
        temps.append(point.temp)
        have_temps = True

  if not have_temps:
    raise ChartError('Not enough data')

  res = {
    'series': [
      {
        'data': temps,
        'marker': {
          'enabled': False,
        },
      },
    ],
    'tooltip': {
      'enabled': False,
    },
    'xAxis': {
      'categories': ['Temperature'],
      'labels': {
        'enabled': False,
      },
      'tickInterval': 0,
    },
    'yAxis': {
      'labels': {
        'enabled': False,
      },
      'tickInterval': 1,
    },
  }
  return res

def KegVolumeChart(keg):
  if not isinstance(keg, models.Keg):
    raise ChartError('Bad value given for keg')
  stats = keg.GetStats()

  served = units.Quantity(stats.get('total_volume_ml', 0))
  served_pints = to_pints(served)
  full_pints = to_pints(keg.full_volume())
  remain_pints = full_pints - served_pints

  res = {
    'chart': {
      'defaultSeriesType': 'bar',
    },
    'series': [
      {'data': [served_pints]},
    ],
    'tooltip': {
      'enabled': False,
    },
    'xAxis': {
      'categories': ['Served'],
      'labels': {
        'enabled': False,
      },
      'gridLineWidth': 0,
    },
    'yAxis': {
      'endOnTick': False,
      'min': 0,
      'max': full_pints,
      'lineWidth': 0,
      'labels': {
        'enabled': False,
      },
    },
  }
  return res

def VolumeByWeekday(stats):
  """ Shows a histogram of volume by day of the week.

  Syntax:
    {% chart volume_by_day <stats> width height %}
  Args:
    stats - a stats object containing volume_by_day_of_week
  """
  volmap = [0] * 7
  vols = stats.get('volume_by_day_of_week', [])
  if not volmap:
    raise ChartError('Daily volumes unavailable')

  for v in vols:
    volmap[int(v.weekday)] += to_pints(v.volume_ml)
  return _DayOfWeekChart(volmap)

def UserSessionsByWeekday(user):
  """Shows a user's total session by volume by day of week."""
  if not isinstance(user, models.User):
    raise ChartError('Bad value for user')
  chunks = user.user_session_chunks.all()
  weekdays = [0] * 7
  for chunk in chunks:
    # Convert from Sunday = 6 to Sunday = 0
    weekday = (chunk.start_time.weekday() + 1) % 7
    weekdays[weekday] += to_pints(chunk.volume_ml)
  return _DayOfWeekChart(weekdays)

def SessionVolumes(sessions):
  buckets = [0]*6
  labels = [
    '<1',
    '1.0-1.9',
    '2.0-2.9',
    '3.0-3.9',
    '4.0-4.9',
    '5+'
  ]
  for sess in sessions:
    pints = round(to_pints(sess.volume_ml), 1)
    intval = int(pints)
    if intval >= len(buckets):
      buckets[-1] += 1
    else:
      buckets[intval] += 1

  res = {
    'xAxis': {
      'categories': labels,
    },
    'series': [
      {'data': buckets},
    ],
    'yAxis': {
      'min': 0,
    },
    'chart': {
      'defaultSeriesType': 'column',
    }
  }
  return res

def UsersByVolume(stats):
  vols = stats.get('volume_by_drinker')
  if not vols:
    raise ChartError('no data')

  volmap = {}
  data = []
  for entry in vols:
    pints = to_pints(entry.volume_ml)
    label = '<b>%s</b> (%.1f)' % (entry.username, pints)
    data.append((label, pints))

  other_vol = 0
  for username, pints in data[10:]:
    other_vol += pints

  def _sort_vol_desc(a, b):
    return cmp(b[1], a[1])

  data.sort(_sort_vol_desc)
  data = data[:10]
  data.reverse()

  if other_vol:
    label = '<b>%s</b> (%.1f)' % ('all others', other_vol)
    data.append((label, other_vol))

  res = {
    'series': [
      {
        'type': 'pie',
        'name': 'Drinkers by Volume',
        'data': data,
      }
    ],
    'yAxis': {
      'min': 0,
    },
    'chart': {
      'defaultSeriesType': 'column',
    },
    'tooltip': {
      'enabled': False,
    },
  }
  return res

def UserSessionChunks(user_chunk):
  if not isinstance(user_chunk, models.UserSessionChunk):
    raise ChartUnavailableError, "Must give user chunk as argument"

  max_pints = user_chunk.session.UserChunksByVolume()
  if not max_pints:
    raise ChartUnavailableError, "Error: session corrupt"
  max_pints = to_pints(max_pints[0].volume_ml)
  drinks = user_chunk.GetDrinks()
  totals = {}
  for drink in drinks:
    if drink.keg:
      label = 'keg %i' % drink.keg.seqn
    else:
      label = 'unknown keg'
    totals[label] = totals.get(label, 0) + to_pints(drink.volume_ml)

  series = []
  for name, tot in totals.iteritems():
    series.append({
      'name': name,
      'data': [tot],
    })

  res = {
    'xAxis': {
      'categories': ['Pints'],
    },
    'yAxis': {
      'max': max_pints,
    },
    'series': series,
    'tooltip': {
      'enabled': False,
    },
    'chart': {
      'defaultSeriesType': 'bar',
    },
    'legend': {
      'enabled': True,
    },
    'plotOptions': {
      'series': {
        'stacking': 'normal',
      },
    },
  }
  return res

def _DayOfWeekChart(vals):
  labels = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat']

  # convert from 0=Monday to 0=Sunday
  #vals.insert(0, vals.pop(-1))

  res = {
    'xAxis': {
      'categories': labels,
    },
    'yAxis': {
      'min': 0,
    },
    'series': [
      {'data': vals},
    ],
    'tooltip': {
      'enabled': False,
    },
    'chart': {
      'defaultSeriesType': 'column',
    }
  }
  return res



from pykeg.core import features

from django.conf import settings
from django.conf.urls.defaults import *
from django.contrib import admin
from django.contrib.auth.views import password_reset
from django.contrib.auth.views import password_reset_complete
from django.contrib.auth.views import password_reset_confirm
from django.contrib.auth.views import password_reset_done
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

admin.autodiscover()

try:
  from registration.views import register
  from pykeg.web.kegweb.forms import KegbotRegistrationForm
  USE_DJANGO_REGISTRATION = True
except ImportError:
  USE_DJANGO_REGISTRATION = False

urlpatterns = patterns('',
    ### django admin site
    (r'^admin/', include(admin.site.urls)),

    (r'^favicon.ico$', 'django.views.generic.simple.redirect_to',
      {'url': '/site_media/images/favicon.ico'}),

    ### RESTful api
    (r'^(?P<kbsite_name>)api/', include('pykeg.web.api.urls')),

    ### account
    (r'^account/', include('pykeg.web.account.urls')),

    (r'^accounts/', include('pykeg.web.registration.urls')),
    url(r'^accounts/password/reset/$', password_reset, {'template_name':
     'registration/password_reset.html'}, name="password-reset"),
    (r'^accounts/password/reset/done/$', password_reset_done, {'template_name':
     'registration/password_reset_done.html'}),
    (r'^accounts/password/reset/confirm/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$', password_reset_confirm, {'template_name':
     'registration/password_reset_confirm.html'}),
    (r'^accounts/password/reset/complete/$', password_reset_complete, {'template_name':
     'registration/password_reset_complete.html'}),

    ### socialregistration
    (r'^sr/', include('socialregistration.urls', namespace='socialregistration')),

    ### charts
    (r'^(?P<kbsite_name>)charts/', include('pykeg.web.charts.urls')),

    ### kegadmin
    (r'^(?P<kbsite_name>)kegadmin/', include('pykeg.web.kegadmin.urls')),
)

if features.use_facebook():
  urlpatterns += patterns('',
      ### facebook kegweb stuff
      (r'^(?P<kbsite_name>)fb/', include('pykeg.web.contrib.facebook.urls')),
  )

### accounts and registration
# uses the stock django-registration views, except we need to override the
# registration class for acocunt/register
if USE_DJANGO_REGISTRATION:
  from django.contrib.auth import views as auth_views
  urlpatterns += patterns('',
    url(r'^(?P<kbsite_name>)accounts/register/$', register,
      {'form_class':KegbotRegistrationForm},
      name='registration_register',
    ),
   (r'^accounts/', include('registration.urls')),
  )

if settings.DEBUG:
  urlpatterns += staticfiles_urlpatterns()
  urlpatterns += patterns('',
    url(r'^media/(?P<path>.*)$', 'django.views.static.serve', { 'document_root': settings.MEDIA_ROOT, }),
  )

### Raven
if settings.HAVE_SENTRY:
  urlpatterns += patterns('',
      (r'^sentry/', include('sentry.web.urls')),
  )

### main kegweb urls
urlpatterns += patterns('',
  (r'^(?P<kbsite_name>)', include('pykeg.web.kegweb.urls')),
)



# Copyright 2011 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import random

class ApiKey:
  """
  Format
    VRRRUUUUSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS
    V = Version (1 hex digit)
    R = Reserved (3 hex digits, zeroes)
    U = UID (4 hex digits)
    S = Secret (32 chars)
  40 characters
  """
  VERSION = 1

  def __init__(self, uid, secret=None):
    self._uid = uid
    if secret is None:
      secret = ApiKey.NewSecret()
    self._secret = secret

  def __str__(self):
    return self.Encode()

  def Encode(self):
    return ApiKey._EncodeParts(self._uid, self._secret)

  def uid(self):
    return self._uid

  def secret(self):
    return self._secret

  @classmethod
  def FromString(cls, s):
    if s is None:
      raise ValueError("Must provide a string.")

    if len(s) != 40:
      raise ValueError("Bad key length.")

    version = s[0]
    reserved = s[1:4]
    uidstr = s[4:8]
    secret = s[8:]

    if version != str(ApiKey.VERSION):
      raise ValueError("Bad key version.")
    if reserved != '000':
      raise ValueError("Illegal value for reserved field: '%s'" % reserved)
    uid = int(uidstr, 16)

    return ApiKey(uid, secret)

  @classmethod
  def NewSecret(cls):
    return '%032x' % random.randint(0, 2**128 - 1)

  @classmethod
  def _EncodeParts(cls, uid, secret):
    if len(secret) != 32:
      raise ValueError, "Bad secret length"
    result = '%01i%03i%04x%s' % (cls.VERSION, 0, uid, secret)
    return result




#!/usr/bin/env python
#
# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import cStringIO
import datetime

from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.views.generic.simple import redirect_to
from django.forms import widgets
from django.views.decorators.http import require_POST

from pykeg.core import backup
from pykeg.core import models
from pykeg.connections.foursquare import forms as foursquare_forms
from pykeg.connections.foursquare import models as foursquare_models
from pykeg.connections.twitter import models as twitter_models
from pykeg.connections.twitter import forms as twitter_forms
from pykeg.connections.twitter import util as twitter_util

from pykeg.web.kegadmin import forms

@staff_member_required
def kegadmin_main(request):
  context = RequestContext(request)
  form = forms.SiteSettingsForm(instance=request.kbsite.settings)
  if request.method == 'POST':
    form = forms.SiteSettingsForm(request.POST, instance=request.kbsite.settings)
    if form.is_valid():
      form.save()
      messages.success(request, 'Site settings were successfully updated.')
  context['settings_form'] = form
  return render_to_response('kegadmin/index.html', context)

def _create_or_update_tap(tap, request):
  if tap:
    prefix = 'tap-%s' % (tap.seqn,)
  else:
    prefix = None
  form = forms.TapForm(request.POST, site=request.kbsite, instance=tap,
      prefix=prefix)
  if form.is_valid():
    new_tap = form.save(commit=False)
    new_tap.site = request.kbsite
    val = form.cleaned_data['ml_per_tick']
    ml = val[0]
    if ml == '0':
      ml = val[1]
    new_tap.ml_per_tick = float(ml)
    new_tap.save()
    if tap:
      messages.success(request, 'Tap "%s" was updated.' % new_tap.name)
    else:
      messages.success(request, 'Tap "%s" was created.' % new_tap.name)
    return True, form
  messages.error(request, 'The tap form had errors, please correct them.')
  return False, form

@staff_member_required
def tap_list(request):
  context = RequestContext(request)

  # Create a new tap if needed.
  context['create_tap_form'] = forms.TapForm(site=request.kbsite)
  if request.method == 'POST' and 'new-tap-submit' in request.POST:
    success, form = _create_or_update_tap(None, request)
    if not success:
      context['create_tap_form'] = form

  for tap in request.kbsite.taps.all():
    submit_name = 'tap-%s-delete' % tap.seqn
    if request.method == 'POST' and submit_name in request.POST:
      tap_name = tap.name
      tap.delete()
      messages.success(request, 'Tap "%s" was deleted.' % tap_name)

  # Build/process forms for all taps.
  taps = [{'tap': tap} for tap in request.kbsite.taps.all()]
  for tinfo in taps:
    tap = tinfo['tap']
    prefix = 'tap-%s' % tap.seqn
    tinfo['prefix'] = prefix

    # Demux multiple forms using submit button name.
    submit_name = "%s-submit" % prefix
    if request.method == 'POST' and submit_name in request.POST:
      success, form = _create_or_update_tap(tap, request)
      if not success:
        tinfo['edit_form'] = form

    if 'edit_form' not in tinfo:
      tinfo['edit_form'] = forms.TapForm(site=request.kbsite, instance=tap,
          prefix=prefix, initial={'ml_per_tick': (str(tap.ml_per_tick), str(tap.ml_per_tick))})

    if tap.current_keg and tap.current_keg.is_active():
      tinfo['end_form'] = forms.KegHiddenSelectForm(initial={'keg':tap.current_keg})
    else:
      tinfo['keg_form'] = forms.ChangeKegForm()

  context['all_taps'] = taps
  return render_to_response('kegadmin/tap-edit.html', context)

@staff_member_required
def connections(request):
  context = RequestContext(request)
  kbsite = request.kbsite

  try:
    twitter_profile = request.kbsite.twitter_profile
  except twitter_models.SiteTwitterProfile.DoesNotExist:
    twitter_profile = None

  tweet_form = forms.TweetForm()
  if request.method == 'POST' and 'tweet-form-submit' in request.POST:
    tweet_form = forms.TweetForm(request.POST)
    if tweet_form.is_valid():
      if kbsite.twitter_profile.is_enabled():
        tweet = tweet_form.cleaned_data['tweet']
        twitter_util.update_site_status(kbsite, tweet)
        tweet_form = forms.TweetForm()
        messages.success(request, 'Tweetz0red!')
      else:
        messages.error(request, 'Twitter profile not enabled.')

  if twitter_profile:
    instance, _ = twitter_models.SiteTwitterSettings.objects.get_or_create(profile=twitter_profile)
    twitter_settings_form = twitter_forms.SiteTwitterSettingsForm(instance=instance)
    if request.method == 'POST' and 'twitter-settings-submit' in request.POST:
      twitter_settings_form = twitter_forms.SiteTwitterSettingsForm(request.POST, instance=instance)
      if twitter_settings_form.is_valid():
        twitter_settings_form.save()
      else:
        messages.error(request, 'Twitter settings form had errors.')
    context['twitter_settings_form'] = twitter_settings_form

  instance, _ = foursquare_models.SiteFoursquareSettings.objects.get_or_create(site=kbsite)
  foursquare_form = foursquare_forms.SiteFoursquareSettingsForm(instance=instance)
  if request.method == 'POST' and 'foursquare-settings-submit' in request.POST:
    foursquare_form = foursquare_forms.SiteFoursquareSettingsForm(request.POST, instance=instance)
    if foursquare_form.is_valid():
      foursquare_form.save()
      messages.success(request, 'Foursquare settings updated!')

  context['twitter_profile'] = twitter_profile
  context['tweet_form'] = tweet_form
  context['foursquare_settings_form'] = foursquare_form
  return render_to_response('kegadmin/connections.html', context)

@staff_member_required
@require_POST
def do_end_keg(request):
  form = forms.KegHiddenSelectForm(request.POST)
  if form.is_valid():
    keg = form.cleaned_data['keg']
    if keg.site == request.kbsite:
      keg.status = "offline"
      keg.end_time = datetime.datetime.now()
      keg.save()
      if keg.current_tap:
        keg.current_tap.current_keg = None
        keg.current_tap.save()
      messages.success(request, 'The keg was deactivated.')
  result_url = reverse('kegadmin-taps', args=[request.kbsite.url()])
  return redirect_to(request, result_url)

@staff_member_required
@require_POST
def edit_tap(request, tap_id):
  context = RequestContext(request)
  tap = get_object_or_404(models.KegTap, site=request.kbsite, seqn=tap_id)
  form = forms.ChangeKegForm()

  form = forms.ChangeKegForm(request.POST)
  if form.is_valid():
    current = tap.current_keg
    if current and current.status != 'offline':
      current.status = 'offline'
      current.save()
    new_keg = models.Keg()
    new_keg.site = request.kbsite
    new_keg.type = form.cleaned_data['beer_type']
    new_keg.size = form.cleaned_data['keg_size']
    new_keg.status = 'online'
    if form.cleaned_data['description']:
      new_keg.description = form.cleaned_data['description']
    new_keg.save()
    tap.current_keg = new_keg
    tap.save()
    messages.success(request, 'The new keg was activated. Bottoms up!')
    form = forms.ChangeKegForm()

  result_url = reverse('kegadmin-taps', args=[request.kbsite.url()])
  return redirect_to(request, result_url)

@staff_member_required
def create_tap(request):
  context = RequestContext(request)
  form = forms.ChangeKegForm()

@staff_member_required
def edit_keg(request, keg_id):
  context = RequestContext(request)

@staff_member_required
def backup_restore(request):
  context = RequestContext(request)
  return render_to_response('kegadmin/backup-restore.html', context)

@staff_member_required
def generate_backup(request):
  context = RequestContext(request)

  fmt = request.GET.get('fmt', 'minjson')
  if fmt == 'json':
    indent = 2
  else:
    indent = None
    fmt = 'minjson'

  kbsite = request.kbsite
  datestr = datetime.datetime.now().strftime('%Y%m%d-%H%M%S')
  filename = 'kegbot-%s.%s.%s.txt' % (kbsite.name, datestr, fmt)

  output_fp = cStringIO.StringIO()
  backup.dump(output_fp, kbsite, indent=indent)

  response = HttpResponse(output_fp.getvalue(),
      mimetype="application/octet-stream")
  response['Content-Disposition'] = 'attachment; filename=%s' % filename
  output_fp.close()
  return response



from django.conf.urls.defaults import *
from django.views.generic.simple import direct_to_template

from registration.views import activate
from registration.views import register


urlpatterns = patterns('',
                       url(r'^activate/complete/$',
                           direct_to_template,
                           {'template': 'registration/activation_complete.html'},
                           name='registration_activation_complete'),
                       url(r'^activate/(?P<activation_key>\w+)/$',
                           activate,
                           {'backend': 'pykeg.web.registration.KegbotBackend'},
                           name='registration_activate'),
                       url(r'^register/$',
                           register,
                           {'backend': 'pykeg.web.registration.KegbotBackend'},
                           name='registration_register'),
                       url(r'^register/complete/$',
                           direct_to_template,
                           {'template': 'registration/registration_complete.html'},
                           name='registration_complete'),
                       url(r'^register/closed/$',
                           direct_to_template,
                           {'template': 'registration/registration_closed.html'},
                           name='registration_disallowed'),
                       (r'', include('registration.auth_urls')),
                       )



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import admin
from pykeg.beerdb import models

from imagekit.admin import AdminThumbnail

class BeerTypeInline(admin.TabularInline):
  model = models.BeerType

class BrewerAdmin(admin.ModelAdmin):
  inlines = [
      BeerTypeInline,
  ]

admin.site.register(models.Brewer, BrewerAdmin)
admin.site.register(models.BeerStyle)

class BeerTypeAdmin(admin.ModelAdmin):
  list_display = ('name', 'brewer', 'style')
admin.site.register(models.BeerType, BeerTypeAdmin)

class BeerImageAdmin(admin.ModelAdmin):
  model = models.BeerImage
  list_display = ('__str__', 'admin_thumbnail')
  admin_thumbnail = AdminThumbnail(image_field='thumbnail')
admin.site.register(models.BeerImage, BeerImageAdmin)



# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import admin
from . import models
from django.conf import settings
from django.forms import ModelForm, PasswordInput, CharField
from django.core.exceptions import ValidationError
from hashlib import md5

import urllib, urllib2, base64

try:
  import json
except ImportError:
  try:
    import simplejson as json
  except ImportError:
    try:
      from django.utils import simplejson as json
    except ImportError:
      raise ImportError, "Unable to load a json library"


class UserUntappdLinkAdminForm(ModelForm):
  password = CharField(label='Untappd Password',  widget=PasswordInput, required=True)

  class Meta:
    model = models.UserUntappdLink

  def __init__(self, *args, **kwargs):
    super(UserUntappdLinkAdminForm, self).__init__(*args, **kwargs)

  def clean(self):
    cleaned_data = super(UserUntappdLinkAdminForm, self).clean()
    
    self.password_hash = md5(cleaned_data['password']).hexdigest()
    
    if settings.UNTAPPD_API_KEY == None:
      raise ValidationError('Untappd API key is not set. Please add your UNTAPPD_API_KEY to your local_settings.py file.')
    
    req = urllib2.Request("http://api.untappd.com/v3/user?key=" + settings.UNTAPPD_API_KEY)
    
    base64string = base64.encodestring('%s:%s' % (cleaned_data['untappd_name'], self.password_hash))[:-1]
    authheader =  "Basic %s" % base64string
    req.add_header("Authorization", authheader)
            
    try:
      response = urllib2.urlopen(req)
    
    except urllib2.HTTPError:
      self.error_messages = {'response_error' : 'There was a problem validating the password with Untappd. Please check the username / password or try again later.'}
      raise ValidationError(self.error_messages['response_error'])
              
    try:
      response = json.loads(response.read())
      
      if response.has_key('error'):
        self.error_messages = {'response_error' : str(response['error'])}
        raise ValidationError(self.error_messages['response_error'])
    
    except KeyError, ValueError:
      self.error_messages = {'response_error' : 'There was a problem validating the password with Untappd. Try again later.'}
      raise ValidationError(self.error_messages['response_error'])
    
    return cleaned_data

  def save(self, commit=True):    
    model = super(UserUntappdLinkAdminForm, self).save(commit=False)
    model.untappd_password_hash = self.password_hash
      
    if commit:
      model.save()
    
    return model

class UserUntappdLinkAdmin(admin.ModelAdmin):
  def get_readonly_fields(self, request, obj=None):
    return self.readonly_fields + ('untappd_password_hash',)
    
  list_display = ('untappd_name', 'user_profile')
  form = UserUntappdLinkAdminForm
    

admin.site.register(models.UserUntappdLink, UserUntappdLinkAdmin)




### Drink-related constants

# Don't record teeny drinks
MIN_VOLUME_TO_RECORD = 10

# Don't show small drinks
MIN_VOLUME_TO_DISPLAY = 10

# Minimum session size, in milliliters, to display
MIN_SESSION_VOLUME_DISPLAY_ML = 177  # 6 ounces

# Maximum time between consecutive drinks to be considered in the same 'session'
# (see UserDrinkingSession table)
DRINK_SESSION_TIME_MINUTES = 3 * 60

# The maximum difference between consecutive meter readings that is considered
# valid.
MAX_METER_READING_DELTA = 2200*2

# Minimum and maximum thermo sensor readings (degrees C).
THERMO_SENSOR_RANGE = (-20.0, 80.0)

# Address the kegnet server should bind to.
KB_CORE_DEFAULT_ADDR = 'localhost:9805'

DEFAULT_NEW_USER_WEIGHT = 140
DEFAULT_NEW_USER_GENDER = 'male'

# String name for all taps
ALIAS_ALL_TAPS = '__all_taps__'

# Device names
AUTH_MODULE_CORE_ONEWIRE = 'core.onewire'
AUTH_MODULE_CORE_RFID = 'core.rfid'
AUTH_MODULE_CONTRIB_PHIDGET_RFID = AUTH_MODULE_CORE_RFID

# Auth modules whose token values should be interpreted as lower-case hex.
AUTH_MODULE_NAMES_HEX_VALUES = (AUTH_MODULE_CORE_ONEWIRE, AUTH_MODULE_CORE_RFID)

# Flag which determines whether an auth device is captive or non-captive.  A
# captive device is one which captures the authentication token, and provides a
# very reliable signal when the token is detached.
#
# For a device marked as captive, the AuthenticationManager will immediately end
# any active flows when a token is removed.  For non-captive (or contactless)
# devices, such as an RFID reader, the authentication manager does nothing when
# the token is removed (see flow timeout, next).
AUTH_DEVICE_CAPTIVE = {
  AUTH_MODULE_CORE_ONEWIRE: True,
  AUTH_MODULE_CORE_RFID: False,
  'default': True
}

# Maximum idle time for new flows, based on initiating auth device.  "Idle" is
# defined as seconds elapsed without any flow meter activity.
#
# This varies on a per-auth-device basis due to the distinction between captive
# and non-captive devices: we want flows initiated with a contactless auth
# device, like an RFID, to timeout sooner.
AUTH_DEVICE_MAX_IDLE_SECS = {
  AUTH_MODULE_CORE_ONEWIRE: 120,
  AUTH_MODULE_CORE_RFID: 20,
  'default': 10
}

# How often to record a thermo reading?
THERMO_RECORD_DELTA_SECONDS = 60

### Addresses and ports

# Default port for JSON server
JSON_SERVER_PORT_DEFAULT = 9100

### Exceptions

class ConfigurationError(Exception):
  """Raised when the system is misconfigured"""



# Copyright 2011 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.core.management.base import BaseCommand
from django.core.management.base import CommandError

import datetime

from pykeg.core import models


class Command(BaseCommand):
  help = u'Deletes any old temperature sensor records'
  args = '<none>'

  def handle(self, *args, **options):
    if len(args) != 0:
      raise CommandError('No arguments required')

    now = datetime.datetime.now()

    # keep at least the most recent 24 hours
    keep_time = now - datetime.timedelta(hours=24)

    # round down to the start of the day
    keep_date = datetime.datetime(year=keep_time.year, month=keep_time.month,
        day=keep_time.day)

    print "Deleting entries older than %s" % keep_date

    for sensor in models.ThermoSensor.objects.all():
      old_entries = sensor.thermolog_set.filter(time__lt=keep_date).order_by('time')
      print "Sensor: %s" % sensor
      print "Deleting %s entries ..." % (old_entries.count())
      old_entries.delete()
      print ""



# Copyright 2011 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from optparse import make_option

from django.core.management.base import BaseCommand
from django.core.management.base import CommandError

from pykeg.core import defaults

class Command(BaseCommand):
  help = u'Set defaults in a new kegbot database.'
  args = '<none>'
  option_list = BaseCommand.option_list + (
      make_option('-f', '--force',
        action='store_true',
        dest='force',
        default=False,
        help='Forcibly set defaults (no database check).'),)

  def handle(self, *args, **options):
    if len(args) != 0:
      raise CommandError('No arguments required')

    try:
      defaults.set_defaults(options['force'])
    except RuntimeError:
      print 'Error: database already installed, defaults cannot be set'
      return



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):
    
    def forwards(self, orm):
        
        # Adding model 'Brewer'
        db.create_table('beerdb_brewer', (
            ('edited', self.gf('django.db.models.fields.DateTimeField')()),
            ('added', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('url', self.gf('django.db.models.fields.URLField')(default='', max_length=200, null=True, blank=True)),
            ('country', self.gf('pykeg.core.fields.CountryField')(default='USA', max_length=3)),
            ('description', self.gf('django.db.models.fields.TextField')(default='', null=True, blank=True)),
            ('production', self.gf('django.db.models.fields.CharField')(default='commercial', max_length=128)),
            ('origin_city', self.gf('django.db.models.fields.CharField')(default='', max_length=128, null=True, blank=True)),
            ('origin_state', self.gf('django.db.models.fields.CharField')(default='', max_length=128, null=True, blank=True)),
            ('id', self.gf('django.db.models.fields.CharField')(max_length=36, primary_key=True)),
            ('revision', self.gf('django.db.models.fields.IntegerField')(default=1)),
        ))
        db.send_create_signal('beerdb', ['Brewer'])

        # Adding model 'BeerStyle'
        db.create_table('beerdb_beerstyle', (
            ('edited', self.gf('django.db.models.fields.DateTimeField')()),
            ('added', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('id', self.gf('django.db.models.fields.CharField')(max_length=36, primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('revision', self.gf('django.db.models.fields.IntegerField')(default=1)),
        ))
        db.send_create_signal('beerdb', ['BeerStyle'])

        # Adding model 'BeerType'
        db.create_table('beerdb_beertype', (
            ('edited', self.gf('django.db.models.fields.DateTimeField')()),
            ('style', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['beerdb.BeerStyle'])),
            ('added', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('brewer', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['beerdb.Brewer'])),
            ('carbs_oz', self.gf('django.db.models.fields.FloatField')(null=True, blank=True)),
            ('edition', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('calories_oz', self.gf('django.db.models.fields.FloatField')(null=True, blank=True)),
            ('abv', self.gf('django.db.models.fields.FloatField')(null=True, blank=True)),
            ('specific_gravity', self.gf('django.db.models.fields.FloatField')(null=True, blank=True)),
            ('original_gravity', self.gf('django.db.models.fields.FloatField')(null=True, blank=True)),
            ('id', self.gf('django.db.models.fields.CharField')(max_length=36, primary_key=True)),
            ('revision', self.gf('django.db.models.fields.IntegerField')(default=1)),
        ))
        db.send_create_signal('beerdb', ['BeerType'])
    
    
    def backwards(self, orm):
        
        # Deleting model 'Brewer'
        db.delete_table('beerdb_brewer')

        # Deleting model 'BeerStyle'
        db.delete_table('beerdb_beerstyle')

        # Deleting model 'BeerType'
        db.delete_table('beerdb_beertype')
    
    
    models = {
        'beerdb.beerstyle': {
            'Meta': {'object_name': 'BeerStyle'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '1'})
        },
        'beerdb.beertype': {
            'Meta': {'object_name': 'BeerType'},
            'abv': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'brewer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['beerdb.Brewer']"}),
            'calories_oz': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'carbs_oz': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'edition': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'original_gravity': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'specific_gravity': ('django.db.models.fields.FloatField', [], {'null': 'True', 'blank': 'True'}),
            'style': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['beerdb.BeerStyle']"})
        },
        'beerdb.brewer': {
            'Meta': {'object_name': 'Brewer'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'country': ('pykeg.core.fields.CountryField', [], {'default': "'USA'", 'max_length': '3'}),
            'description': ('django.db.models.fields.TextField', [], {'default': "''", 'null': 'True', 'blank': 'True'}),
            'edited': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.CharField', [], {'max_length': '36', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'origin_city': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'origin_state': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'production': ('django.db.models.fields.CharField', [], {'default': "'commercial'", 'max_length': '128'}),
            'revision': ('django.db.models.fields.IntegerField', [], {'default': '1'}),
            'url': ('django.db.models.fields.URLField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'})
        }
    }
    
    complete_apps = ['beerdb']



from django import forms

from pykeg.core import models
from pykeg.beerdb import models as bdb

from pykeg.web.kegadmin import widgets as ka_widgets

ALL_TAPS = models.KegTap.objects.all()
ALL_SIZES = models.KegSize.objects.all()
ALL_BEER_TYPES = bdb.BeerType.objects.all().order_by('name')
ALL_KEGS = models.Keg.objects.all()

class GeneralSettingsForm(forms.Form):
  name = forms.CharField(help_text='Name of this Kegbot system')

class ChangeKegForm(forms.Form):
  keg_size = forms.ModelChoiceField(queryset=ALL_SIZES,
      empty_label=None,
      help_text='Size of the new keg')

  beer_type = forms.ModelChoiceField(queryset=ALL_BEER_TYPES,
      empty_label=None,
      help_text='Choose existing type')

  cost = forms.FloatField(required=False, min_value=0,
      help_text='Price paid (optional)')
  description = forms.CharField(required=False,
      help_text='Public description of this specific keg (optional)')

class TapForm(forms.ModelForm):
  class Meta:
    model = models.KegTap

class KegHiddenSelectForm(forms.Form):
  keg = forms.ModelChoiceField(queryset=ALL_KEGS, widget=forms.HiddenInput)

class TapForm(forms.ModelForm):
  class Meta:
    model = models.KegTap
    fields = ('name', 'meter_name', 'relay_name', 'description',
        'temperature_sensor')

  ml_per_tick = ka_widgets.ChoiceWithOtherField(
      label='Meter type',
      choices=(
        (1/2.2, 'Vision 2000 Meter (%s mL/tick)' % (1/2.2) ),
        (1/6.0, 'SwissFlow Meter (%s mL/tick)' % (1/6.0) ),
        (0, 'Other (specify in mL/tick):'),
      ),
  )

  def __init__(self, *args, **kwargs):
    site = kwargs.pop('site', None)
    super(TapForm, self).__init__(*args, **kwargs)
    self.fields['temperature_sensor'].queryset = models.ThermoSensor.objects.filter(site=site)
    self.fields['temperature_sensor'].empty_label = 'No sensor.'

class SiteSettingsForm(forms.ModelForm):
  class Meta:
    model = models.SiteSettings
    fields = (
        'title',
        'description',
        'privacy',
        'display_units',
        'event_web_hook',
        'session_timeout_minutes',
        'google_analytics_id',
        'guest_name',
        'guest_image',
        'default_user',
        'registration_allowed',
        'registration_confirmation',
    )

#BeerTypeFormSet = inlineformset_factory(models.Brewer, models.BeerType)

class TweetForm(forms.Form):
  tweet = forms.CharField(max_length=140, required=True)



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegweb API client."""

import datetime
import functools
import socket
import sys
import types
import urllib2

from kegbot.util import kbjson

import urllib2
from urllib import urlencode
from urllib2 import HTTPError
from urllib2 import URLError

import gflags

gflags.DEFINE_float('krest_timeout', 10.0,
    'Socket timeout, in seconds, for Kegbot web API operations. '
    'Note that this timeout only applies to blocking socket operations '
    '(such as opening a connection) and not I/O.')

FLAGS = gflags.FLAGS

_DEFAULT_URL = 'http://localhost:8000/api/'
_DEFAULT_KEY = ''
try:
  from pykeg import settings
  if hasattr(settings, 'KEGWEB_BASE_URL'):
    _DEFAULT_URL = '%s/api/' % getattr(settings, 'KEGWEB_BASE_URL')
  if hasattr(settings, 'KEGWEB_API_KEY'):
    _DEFAULT_KEY = settings.KEGWEB_API_KEY
except ImportError:
  # Non-fatal if we can't load settings.
  pass

gflags.DEFINE_string('api_url', _DEFAULT_URL,
    'Base URL for the Kegweb HTTP api.')

gflags.DEFINE_string('api_key', _DEFAULT_KEY,
    'Access key for the Kegweb HTTP api.')

### begin common

class Error(Exception):
  """An error occurred."""
  HTTP_CODE = 400
  def Message(self):
    if self.message:
      return self.message
    m = self.__class__.__doc__
    m = m.split('\n', 1)[0]
    return m

class NotFoundError(Error):
  """The requested object could not be found."""
  HTTP_CODE = 404

class ServerError(Error):
  """The server had a problem fulfilling your request."""
  HTTP_CODE = 500

class BadRequestError(Error):
  """The request was incompleted or malformed."""
  HTTP_CODE = 401

class NoAuthTokenError(Error):
  """An api_key is required."""
  HTTP_CODE = 401

class BadApiKeyError(Error):
  """The api_key given is invalid."""
  HTTP_CODE = 401

class PermissionDeniedError(Error):
  """The api_key given does not have permission for this resource."""
  HTTP_CODE = 401

MAP_NAME_TO_EXCEPTION = dict((c.__name__, c) for c in Error.__subclasses__())

def ErrorCodeToException(code, message=None):
  cls = MAP_NAME_TO_EXCEPTION.get(code, Error)
  return cls(message)

def decode_response(response_data):
  """Decodes the string `response_data` as a JSON response.

  For normal responses, the return value is the Python JSON-decoded 'result'
  field of the response.  If the response is an error, a RemoteError exception
  is raised.
  """
  # Decode JSON.
  try:
    d = kbjson.loads(response_data)
  except ValueError, e:
    raise ServerError('Malformed response: %s' % e)

  if 'error' in d:
    # Response had an error: translate to exception.
    err = d['error']
    code = err.get('code', 500)
    message = err.get('message', None)
    e = ErrorCodeToException(code, message)
    raise e
  elif 'object' in d or 'objects' in d:
    # Response was OK, return the result.
    return d
  else:
    # WTF?
    raise ValueError('Invalid response from server: missing result or error')

### end common

class KrestClient:
  """Kegweb RESTful API client."""
  def __init__(self, api_url=None, api_key=None):
    if api_url is None:
      api_url = FLAGS.api_url
    if api_key is None:
      api_key = FLAGS.api_key
    self._api_url = api_url
    self._api_key = api_key
    self._opener = urllib2.build_opener(KrestHTTPErrorProcessor)

  def _Encode(self, s):
    return unicode(s).encode('utf-8')

  def _EncodePostData(self, post_data):
    if not post_data:
      return None
    return urlencode(dict(((k, self._Encode(v)) for k, v in
        post_data.iteritems() if v is not None)))

  def _GetURL(self, endpoint, params=None):
    param_str = ''
    if params:
      param_str = '?%s' % urlencode(params)

    base = self._api_url.rstrip('/')

    # Strip both leading and trailing slash from endpoint: single leading and
    # trailing slashes will be added by the string formatter.  (The trailing
    # slash is required for POSTs to Django.)
    endpoint = endpoint.strip('/')
    return '%s/%s/%s' % (base, endpoint, param_str)

  def SetAuthToken(self, api_key):
    self._api_key = api_key

  def DoGET(self, endpoint, params=None):
    """Issues a GET request to the endpoint, and retuns the result.

    Keyword arguments are passed to the endpoint as GET arguments.

    For normal responses, the return value is the Python JSON-decoded 'object'
    or 'objects' field of the response.  If the response is an error, a
    RemoteError exception is raised.

    If there was an error contacting the server, or in parsing its response, a
    ServerError is raised.
    """
    return self._FetchResponse(endpoint, params=params)

  def DoPOST(self, endpoint, post_data, params=None):
    """Issues a POST request to the endpoint, and returns the result.

    For normal responses, the return value is the Python JSON-decoded 'object'
    or 'objects' field of the response.  If the response is an error, a
    RemoteError exception is raised.

    If there was an error contacting the server, or in parsing its response, a
    ServerError is raised.
    """
    return self._FetchResponse(endpoint, params=params, post_data=post_data)

  def _FetchResponse(self, endpoint, params=None, post_data=None):
    """Issues a POST or GET request, depending on the arguments."""
    if params is None:
      params = {}
    else:
      params = params.copy()

    # If we have an api token, attach it.  Prefer to attach it to POST data, but
    # use GET if there is no POST data.
    if self._api_key:
      if post_data:
        post_data['api_key'] = self._api_key
      else:
        params['api_key'] = self._api_key

    url = self._GetURL(endpoint, params=params)
    encoded_post_data = self._EncodePostData(post_data)

    try:
      # Issue a GET or POST (urlopen will decide based on encoded_post_data).
      response_data = self._opener.open(url, data=encoded_post_data,
          timeout=FLAGS.krest_timeout).read()
    except HTTPError, e:
      raise ServerError('Caused by: %s' % e)
    except URLError, e:
      raise ServerError('URL Error, reason: %s' % e.reason)

    return decode_response(response_data)

  def RecordDrink(self, tap_name, ticks, volume_ml=None, username=None,
      pour_time=None, duration=0, auth_token=None, spilled=False, shout=''):
    endpoint = '/taps/%s' % tap_name
    post_data = {
      'tap_name': tap_name,
      'ticks': ticks,
      'volume_ml': volume_ml,
      'username': username,
      'auth_token': auth_token,
      'duration': duration,
      'spilled': spilled,
    }
    if pour_time:
      post_data['pour_time'] = int(pour_time.strftime('%s'))
      post_data['now'] = int(datetime.datetime.now().strftime('%s'))
    if shout:
      post_data['shout'] = shout
    return self.DoPOST(endpoint, post_data=post_data).object

  def CancelDrink(self, seqn, spilled=False):
    endpoint = '/cancel-drink'
    post_data = {
      'id': seqn,
      'spilled': spilled,
    }
    return self.DoPOST(endpoint, post_data=post_data).object

  def LogSensorReading(self, sensor_name, temperature, when=None):
    endpoint = '/thermo-sensors/%s' % (sensor_name,)
    post_data = {
      'temp_c': float(temperature),
    }
    # TODO(mikey): include post data
    return self.DoPOST(endpoint, post_data=post_data).object

  def TapStatus(self):
    """Gets the status of all taps."""
    return self.DoGET('taps').objects

  def GetToken(self, auth_device, token_value):
    url = 'auth-tokens/%s/%s' % (auth_device, token_value)
    try:
      return self.DoGET(url).object
    except ServerError, e:
      raise NotFoundError(e)

  def LastDrinks(self):
    """Gets a list of the most recent drinks."""
    return self.DoGET('last-drinks').objects

  def AllDrinks(self):
    """Gets a list of all drinks."""
    return self.DoGET('drinks').objects

  def AllSoundEvents(self):
    """Gets a list of all drinks."""
    return self.DoGET('sound-events').objects

class KrestHTTPErrorProcessor(urllib2.BaseHandler):
  def _handle_error(self, request, response, code, msg, hdrs):
    data = response.read()
    decode_response(data)

  http_error_401 = _handle_error
  http_error_403 = _handle_error
  http_error_404 = _handle_error
  http_error_405 = _handle_error
  http_error_500 = _handle_error

def main():
  c = KrestClient()

  print '== record a drink =='
  print c.RecordDrink('kegboard.flow0', 2200)
  print ''

  print '== tap status =='
  for t in c.TapStatus().taps:
    print t
    print ''

  print '== last drinks =='
  for d in c.LastDrinks().drinks:
    print d
    print ''

if __name__ == '__main__':
  FLAGS(sys.argv)
  main()



# Copyright 2011 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from optparse import make_option
import sys

from django.core.management.base import NoArgsCommand
from django.core.management.base import CommandError
from django.contrib.auth.management.commands import createsuperuser
from django.db.utils import DatabaseError

from south.management.commands import migrate
from south.management.commands import syncdb
from pykeg.core.management.commands import kb_set_defaults

from pykeg.core.backend import backend
from pykeg.core.backend.django import KegbotBackend
from pykeg.core import defaults


class Command(NoArgsCommand):
  help = u'Run basic Kegbot setup.'

  def handle(self, **options):
    installed = False
    try:
      b = KegbotBackend()
      installed = defaults.db_is_installed()
    except backend.BackendError, e:
      pass
    except DatabaseError:
      pass

    if installed:
      print 'Error: Kegbot appears to be already installed. ',
      print 'Try "%s kb_upgrade" instead?' % (sys.argv[0],)
      sys.exit(1)

    self.InitialSetup()

  def _RunCommand(self, cmd, args=None):
    if args is None:
      args = []
    cmdname = cmd.__module__.split('.')[-1]
    arg_str = ' '.join('%s' % a for a in args)
    print '--- Running command: %s %s' % (cmdname, arg_str)
    cmd.run_from_argv([sys.argv[0], cmdname] + args)

  def InitialSetup(self):
    # syncdb
    self._RunCommand(syncdb.Command(), args=['--all', '--noinput', '-v', '0'])

    # migrate
    self._RunCommand(migrate.Command(), args=['--all', '--fake', '-v', '0'])

    # kb_set_defaults
    self._RunCommand(kb_set_defaults.Command(), args=['--force'])



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegweb RESTful API views."""

import datetime
from functools import wraps
import logging
import sys
import traceback
from decimal import Decimal
import types

from google.protobuf.message import Message

from django.conf import settings
from django.contrib.auth import login as auth_login
from django.contrib.auth import logout as auth_logout
from django.contrib.auth.forms import AuthenticationForm
from django.db.utils import IntegrityError

from django.http import Http404
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.template.loader import get_template
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.cache import never_cache
from django.db.models.query import QuerySet

from kegbot.api import kbapi
from kegbot.api import protoutil
from kegbot.util import kbjson

from pykeg.contrib.soundserver import models as soundserver_models
from pykeg.core.backend import backend
from pykeg.core.backend.django import KegbotBackend
from pykeg.core import models
from pykeg.proto import protolib
from pykeg.web.api import apikey
from pykeg.web.api import forms

if settings.HAVE_CELERY:
  from pykeg.web import tasks

if settings.HAVE_RAVEN:
  import raven

_LOGGER = logging.getLogger(__name__)

### Decorators

def wrap_exception(f):
  def new_function(*args, **kwargs):
    try:
      return f(*args, **kwargs)
    except Exception, e:
      exc_info = sys.exc_info()
      request = args[0]
      if settings.DEBUG and 'deb' in request.GET:
        raise exc_info[1], None, exc_info[2]
      result_data, http_code = ToJsonError(e, exc_info)
      result_data['meta'] = {
        'result': 'error'
      }
      return HttpResponse(kbjson.dumps(result_data, indent=2),
          mimetype='application/json', status=http_code)
  return wraps(f)(new_function)

def check_api_key(request):
  keystr = request.META.get('HTTP_X_KEGBOT_API_KEY')
  if not keystr:
    keystr = request.REQUEST.get('api_key')
  if not keystr:
    raise kbapi.NoAuthTokenError('The parameter "api_key" is required')

  try:
    key = apikey.ApiKey.FromString(keystr)
  except ValueError, e:
    raise kbapi.BadApiKeyError('Error parsing API key: %s' % e)

  try:
    user = models.User.objects.get(pk=key.uid())
  except models.User.DoesNotExist:
    raise kbapi.BadApiKeyError('API user %s does not exist' % key.uid())

  if not user.is_active:
    raise kbapi.BadApiKeyError('User is inactive')

  if not user.is_staff and not user.is_superuser:
    raise kbapi.PermissionDeniedError('User is not staff/superuser')

  user_secret = user.get_profile().api_secret
  if not user_secret or user_secret != key.secret():
    raise kbapi.BadApiKeyError('User secret does not match')

  request.kb_api_authenticated = True

def auth_required(viewfunc):
  # Set the auth_required flag, validated by
  # pykeg.web.middleware.ApiKeyMiddleware
  viewfunc.kb_api_key_required = True
  def new_function(*args, **kwargs):
    request = args[0]
    if not getattr(request, 'kb_api_authenticated', False):
      check_api_key(request)
    return viewfunc(*args, **kwargs)
  return wraps(viewfunc)(new_function)

def staff_required(viewfunc):
  def _check_token(request, *args, **kwargs):
    if not request.user or (not request.user.is_staff and not \
        request.user.is_superuser):
      raise kbapi.PermissionDeniedError, "Logged-in staff user required"
    return viewfunc(request, *args, **kwargs)
  return wraps(viewfunc)(_check_token)

def ToJsonError(e, exc_info):
  """Converts an exception to an API error response."""
  # Wrap some common exception types into kbapi types
  if isinstance(e, Http404):
    e = kbapi.NotFoundError(e.message)
  elif isinstance(e, ValueError):
    e = kbapi.BadRequestError(str(e))
  elif isinstance(e, backend.NoTokenError):
    e = kbapi.NotFoundError(e.message)

  # Now determine the response based on the exception type.
  if isinstance(e, kbapi.Error):
    code = e.__class__.__name__
    http_code = e.HTTP_CODE
    message = e.Message()
  else:
    code = 'ServerError'
    http_code = 500
    message = 'An internal error occurred: %s' % str(e)
    if settings.DEBUG:
      message += "\n" + "\n".join(traceback.format_exception(*exc_info))
  if settings.DEBUG and settings.HAVE_RAVEN:
    from raven.contrib.django.models import client
    client.captureException()
  result = {
    'error' : {
      'code' : code,
      'message' : message
    }
  }
  return result, http_code

def py_to_json(f):
  """Decorator that wraps an API method.

  The decorator transforms the method in a few ways:
    - The raw return value from the method is converted to a serialized JSON
      result.
    - The result is wrapped in an outer dict, and set as the value 'result'
    - If an exception is thrown during the method, it is converted to a protocol
      error message.
  """
  @never_cache
  @wrap_exception
  def new_function(*args, **kwargs):
    request = args[0]
    http_code = 200
    indent = 2
    if 'indent' in request.GET:
      if request.GET['indent'] == '':
        indent = None
      else:
        try:
          indent_val = int(request.GET['indent'])
          if indent_val >= 0 and indent_val <= 8:
            indent = indent_val
        except ValueError:
          pass
    result_data = prepare_data(f(*args, **kwargs))
    result_data['meta'] = {
      'result': 'ok'
    }
    return HttpResponse(kbjson.dumps(result_data, indent=indent),
        mimetype='application/json', status=http_code)
  return wraps(f)(new_function)

def prepare_data(data, inner=False):
  if isinstance(data, QuerySet) or type(data) == types.ListType:
    result = [prepare_data(d, True) for d in data]
    container = 'objects'
  elif isinstance(data, dict):
    result = data
    container = 'object'
  else:
    result = to_dict(data)
    container = 'object'

  if inner:
    return result
  else:
    return {
      container: result
    }

def to_dict(data):
  if not isinstance(data, Message):
    data = protolib.ToProto(data, full=True)
  return protoutil.ProtoMessageToDict(data)

### Helpers

def _form_errors(form):
  ret = {}
  for field in form:
    if field.errors:
      name = field.html_name
      ret[name] = []
      for error in field.errors:
        ret[name].append(error)
  return ret

### Endpoints

@py_to_json
def all_kegs(request):
  return request.kbsite.kegs.all().order_by('-start_time')

@py_to_json
def all_drinks(request, limit=100):
  qs = request.kbsite.drinks.valid()
  total = len(qs)
  if 'start' in request.GET:
    try:
      start = int(request.GET['start'])
      qs = qs.filter(seqn__lte=start)
    except ValueError:
      pass
  qs = qs.order_by('-seqn')
  qs = qs[:limit]
  start = qs[0].seqn
  count = len(qs)
  return qs

@py_to_json
def get_drink(request, drink_id):
  drink = get_object_or_404(models.Drink, seqn=drink_id, site=request.kbsite)
  return protolib.ToProto(drink, full=True)

@csrf_exempt
@py_to_json
@auth_required
def add_drink_photo(request, drink_id):
  if request.method != 'POST':
    raise Http404('Method not supported')
  drink = get_object_or_404(models.Drink, seqn=drink_id, site=request.kbsite)
  pic = models.Picture.objects.create(site=request.kbsite)
  pic.image = request.FILES['photo']
  pic.drink = drink
  pic.user = drink.user
  pic.keg = drink.keg
  pic.session = drink.session
  pic.save()
  if settings.HAVE_CELERY:
    tasks.handle_new_picture.delay(pic.id)
  return protolib.ToProto(pic, full=True)

@py_to_json
def get_session(request, session_id):
  session = get_object_or_404(models.DrinkingSession, seqn=session_id,
      site=request.kbsite)
  return protolib.ToProto(session, full=True)

@py_to_json
def get_session_stats(request, session_id):
  session = get_object_or_404(models.DrinkingSession, seqn=session_id,
      site=request.kbsite)
  return session.GetStats()

@py_to_json
def get_keg(request, keg_id):
  keg = get_object_or_404(models.Keg, seqn=keg_id, site=request.kbsite)
  return protolib.ToProto(keg, full=True)

@py_to_json
def get_keg_drinks(request, keg_id):
  keg = get_object_or_404(models.Keg, seqn=keg_id, site=request.kbsite)
  return keg.drinks.valid()

@py_to_json
def get_keg_events(request, keg_id):
  keg = get_object_or_404(models.Keg, seqn=keg_id, site=request.kbsite)
  events = keg.events.all()
  events = apply_since(request, events)
  return events

@py_to_json
def all_sessions(request):
  return request.kbsite.sessions.all()

@py_to_json
def current_session(request):
  current = None
  try:
    latest = request.kbsite.sessions.latest()
    if latest.IsActiveNow():
      return latest
  except models.DrinkingSession.DoesNotExist:
    raise Http404

@py_to_json
def all_events(request):
  events = request.kbsite.events.all().order_by('-seqn')
  events = apply_since(request, events)
  events = events[:10]
  return [protolib.ToProto(e, full=True) for e in events]

def apply_since(request, query):
  """Restricts the query to `since` events, if given."""
  since_str = request.GET.get('since')
  if since_str:
    try:
      since = int(since_str)
      return query.filter(seqn__gt=since)
    except (ValueError, TypeError):
      pass
  return query

@py_to_json
@auth_required
def all_sound_events(request):
  return soundserver_models.SoundEvent.objects.all()

@py_to_json
def get_keg_sessions(request, keg_id):
  keg = get_object_or_404(models.Keg, seqn=keg_id, site=request.kbsite)
  sessions = [c.session for c in keg.keg_session_chunks.all()]
  return sessions

@py_to_json
def get_keg_stats(request, keg_id):
  keg = get_object_or_404(models.Keg, seqn=keg_id, site=request.kbsite)
  return keg.GetStatsRecord()

@py_to_json
def get_system_stats(request):
  return request.kbsite.GetStatsRecord()

@py_to_json
def all_taps(request):
  return request.kbsite.taps.all().order_by('name')

@py_to_json
@auth_required
def user_list(request):
  return models.User.objects.filter(is_active=True).order_by('username')

@py_to_json
def get_user(request, username):
  user = get_object_or_404(models.User, username=username)
  return protolib.ToProto(user, full=True)

@py_to_json
def get_user_drinks(request, username):
  user = get_object_or_404(models.User, username=username)
  return user.drinks.valid()

@py_to_json
def get_user_events(request, username):
  user = get_object_or_404(models.User, username=username)
  return user.events.all()

@py_to_json
def get_user_stats(request, username):
  user = get_object_or_404(models.User, username=username)
  return user.get_profile().GetStatsRecord()

@py_to_json
@auth_required
def get_auth_token(request, auth_device, token_value):
  b = KegbotBackend(site=request.kbsite)
  tok = b.GetAuthToken(auth_device, token_value)
  return tok

@csrf_exempt
@py_to_json
@auth_required
def assign_auth_token(request, auth_device, token_value):
  if not request.POST:
    raise kbapi.BadRequestError('POST required.')

  form = forms.AssignTokenForm(request.POST)
  if not form.is_valid():
    errors = _form_errors(form)
    raise kbapi.BadRequestError(errors)

  b = KegbotBackend(site=request.kbsite)
  username = form.cleaned_data['username']
  try:
    tok = b.GetAuthToken(auth_device, token_value)
    user = b._GetUserObjFromUsername(username)
    if not user:
      raise kbapi.BadRequestError("User does not exist")
    if tok.user != user:
      if tok.user:
        raise kbapi.BadRequestError("Token is already bound to a user")
      tok.user = user
      tok.save()
    return tok
  except backend.NoTokenError:
    return b.CreateAuthToken(auth_device, token_value, username=username)

@py_to_json
def all_thermo_sensors(request):
  return request.kbsite.thermosensors.all()

def _get_sensor_or_404(request, sensor_name):
  try:
    sensor = models.ThermoSensor.objects.get(site=request.kbsite,
        raw_name=sensor_name)
  except models.ThermoSensor.DoesNotExist:
    try:
      sensor = models.ThermoSensor.objects.get(site=request.kbsite,
          nice_name=sensor_name)
    except models.ThermoSensor.DoesNotExist:
      raise Http404
  return sensor

@csrf_exempt
def get_thermo_sensor(request, sensor_name):
  if request.method == 'POST':
    return _thermo_sensor_post(request, sensor_name)
  else:
    return _thermo_sensor_get(request, sensor_name)

@py_to_json
def _thermo_sensor_get(request, sensor_name):
  sensor = _get_sensor_or_404(request, sensor_name)
  logs = sensor.thermolog_set.all()
  if not logs:
    last_temp = None
    last_time = None
  else:
    last_temp = logs[0].temp
    last_time = logs[0].time
  res = {
    'sensor': to_dict(sensor),
    'last_temp': last_temp,
    'last_time': last_time,
  }
  return res

@py_to_json
@auth_required
def _thermo_sensor_post(request, sensor_name):
  form = forms.ThermoPostForm(request.POST)
  if not form.is_valid():
    raise kbapi.BadRequestError, _form_errors(form)
  cd = form.cleaned_data
  b = KegbotBackend(site=request.kbsite)
  sensor, created = models.ThermoSensor.objects.get_or_create(site=request.kbsite,
      raw_name=sensor_name)
  # TODO(mikey): use form fields to compute `when`
  return b.LogSensorReading(sensor.raw_name, cd['temp_c'])

@py_to_json
def get_thermo_sensor_logs(request, sensor_name):
  sensor = _get_sensor_or_404(request, sensor_name)
  return sensor.thermolog_set.all()[:60*2]

@py_to_json
def get_api_key(request):
  user = request.user
  api_key = ''
  if user and (user.is_staff or user.is_superuser):
    api_key = str(user.get_profile().GetApiKey())
  return {'api_key': api_key}

@csrf_exempt
def tap_detail(request, tap_id):
  if request.method == 'POST':
    return _tap_detail_post(request, tap_id)
  else:
    return _tap_detail_get(request, tap_id)

@py_to_json
def _tap_detail_get(request, tap_id):
  tap = get_object_or_404(models.KegTap, meter_name=tap_id, site=request.kbsite)
  return protolib.ToProto(tap, full=True)

@py_to_json
@auth_required
def _tap_detail_post(request, tap):
  form = forms.DrinkPostForm(request.POST)
  if not form.is_valid():
    raise kbapi.BadRequestError, _form_errors(form)
  cd = form.cleaned_data
  if cd.get('pour_time') and cd.get('now'):
    pour_time = datetime.datetime.fromtimestamp(cd.get('pour_time'))
    now = datetime.datetime.fromtimestamp(cd.get('now'))
    skew = datetime.datetime.now() - now
    pour_time += skew
  else:
    pour_time = None
  duration = cd.get('duration')
  if duration is None:
    duration = 0
  b = KegbotBackend(site=request.kbsite)
  try:
    res = b.RecordDrink(tap_name=tap,
      ticks=cd['ticks'],
      volume_ml=cd.get('volume_ml'),
      username=cd.get('username'),
      pour_time=pour_time,
      duration=duration,
      auth_token=cd.get('auth_token'),
      spilled=cd.get('spilled'),
      shout=cd.get('shout'))
    return protolib.ToProto(res, full=True)
  except backend.BackendError, e:
    raise kbapi.ServerError(str(e))

@csrf_exempt
@py_to_json
@auth_required
def cancel_drink(request):
  #if request.method != 'POST':
  #  raise kbapi.BadRequestError, 'Method not supported at this endpoint'
  #form = forms.DrinkCancelForm(request.POST)
  form = forms.CancelDrinkForm(request.GET)
  if not form.is_valid():
    raise kbapi.BadRequestError, _form_errors(form)
  cd = form.cleaned_data
  b = KegbotBackend(site=request.kbsite)
  try:
    res = b.CancelDrink(seqn=cd.get('id'), spilled=cd.get('spilled', False))
    return protolib.ToProto(res, full=True)
  except backend.BackendError, e:
    raise kbapi.ServerError(str(e))

@csrf_exempt
@py_to_json
def login(request):
  if request.POST:
    form = AuthenticationForm(data=request.POST)
    if form.is_valid():
      auth_login(request, form.get_user())
      if request.session.test_cookie_worked():
        request.session.delete_test_cookie()
      return {'result': 'ok'}
    else:
      raise kbapi.PermissionDeniedError('Login failed.')
  raise kbapi.BadRequestError('POST required.')

@py_to_json
def logout(request):
  auth_logout(request)
  return {'result': 'ok'}

@csrf_exempt
@py_to_json
@auth_required
def register(request):
  if not request.POST:
    raise kbapi.BadRequestError('POST required.')
  form = forms.RegisterForm(request.POST)
  errors = {}
  if not form.is_valid():
    errors = _form_errors(form)
  else:
    username = form.cleaned_data['username']
    try:
      u = models.User()
      u.username = username
      u.email = form.cleaned_data['email']
      u.save()
      u.set_password(form.cleaned_data['password'])
      if 'photo' in request.FILES:
        pic = models.Picture.objects.create(user=u)
        photo = request.FILES['photo']
        pic.image.save(photo.name, photo)
        pic.save()
        profile = u.get_profile()
        profile.mugshot = pic
        profile.save()
      return protolib.ToProto(u, full=True)
    except IntegrityError:
      user_errs = errors.get('username', [])
      user_errs.append('Username not available.')
      errors['username'] = user_errs
  raise kbapi.BadRequestError(errors)

@py_to_json
def default_handler(request):
  raise Http404, "Not an API endpoint: %s" % request.path[:100]

if settings.HAVE_RAVEN and settings.HAVE_SENTRY:
  class LocalRavenClient(raven.Client):
    logger = logging.getLogger('kegbot.api.client.debug')
    def send(self, **kwargs):
      from sentry.models import GroupedMessage
      return GroupedMessage.objects.from_kwargs(**kwargs)

@csrf_exempt
@py_to_json
@auth_required
def debug_log(request):
  if request.method != 'POST' or not settings.HAVE_RAVEN or not settings.HAVE_SENTRY:
    raise Http404('Method not supported')

  form = forms.DebugLogForm(request.POST)
  if not form.is_valid():
    raise kbapi.BadRequestError(_form_errors(form))
  client = LocalRavenClient([])
  message = form.cleaned_data['message']
  ident = client.get_ident(client.create_from_text(message))
  client.send()
  return {'log_id': ident}




# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegbot API implementation of Backend."""

import logging
import socket

from . import backend

from pykeg.core import kb_common
from pykeg.web.api import kbapi

class WebBackend(backend.Backend):
  def __init__(self, api_url=None, api_key=None):
    self._logger = logging.getLogger('api-backend')
    self._client = kbapi.Client(api_url=api_url, api_key=api_key)

  def CreateNewUser(self, username, gender=kb_common.DEFAULT_NEW_USER_GENDER,
      weight=kb_common.DEFAULT_NEW_USER_WEIGHT):
    raise NotImplementedError

  def CreateAuthToken(self, auth_device, token_value, username=None):
    raise NotImplementedError

  def GetAllTaps(self):
    return self._client.TapStatus()

  def RecordDrink(self, tap_name, ticks, volume_ml=None, username=None,
      pour_time=None, duration=0, auth_token=None, spilled=False, shout=''):
    return self._client.RecordDrink(tap_name=tap_name, ticks=ticks,
        volume_ml=volume_ml, username=username, pour_time=pour_time,
        duration=duration, auth_token=auth_token, spilled=spilled,
        shout=shout)

  def CancelDrink(self, seqn, spilled=False):
    return self._client.CancelDrink(seqn, spilled)

  def LogSensorReading(self, sensor_name, temperature, when=None):
    # If the temperature is out of bounds, reject it.
    min_val = kb_common.THERMO_SENSOR_RANGE[0]
    max_val = kb_common.THERMO_SENSOR_RANGE[1]
    if temperature < min_val or temperature > max_val:
      raise ValueError, 'Temperature out of bounds'

    try:
      return self._client.LogSensorReading(sensor_name, temperature, when)
    except kbapi.NotFoundError:
      self._logger.warning('No sensor on backend named "%s"' % (sensor_name,))
      return None
    except kbapi.ServerError:
      self._logger.warning('Server error recording temperature; dropping reading.')
      return None
    except socket.error:
      self._logger.warning('Socket error recording temperature; dropping reading.')
      return None

  def GetAuthToken(self, auth_device, token_value):
    try:
      return self._client.GetToken(auth_device, token_value)
    except kbapi.NotFoundError:
      raise backend.NoTokenError
    except socket.error:
      self._logger.warning('Socket error fetching token; ignoring.')
      raise backend.NoTokenError



# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Celery tasks for Untappd."""

from pykeg.core import models as core_models
from pykeg.connections import common
from pykeg.connections.foursquare import models as foursquare_models
from django.contrib.sites.models import Site
from . import models
from django.conf import settings

from urllib import urlencode
import urllib2
import base64

from celery.task import task

logger = common.get_logger(__name__)

# Available template vars:
#   name         : user name, or @twitter_name if known
#   kb_name      : kegbot system name
#   kb_url       : kegbot system url
#   session_url  : current session url
#   drink_url    : drink url (if any)
#   drink_size   : drink size
#   beer_name    : beer name

DEFAULT_CHECKIN_TEMPLATE = "Automatic checkin courtesy of %(kb_name)s! %(drink_url)s"

@task
def checkin_event(event):
  if event.kind not in ('drink_poured',):
    logger.info('Event not suitable for checkin: %s' % event.kind)
    return
  do_checkin(event)

def _get_vars(event):
  base_url = 'http://%s/%s' % (Site.objects.get_current().domain, event.site.url())
  name = ''
  if event.user:
    name = event.user.username
  session_url = ''
  drink_url = ''

  if event.drink:
    session_url = '%s/%s' % (base_url, event.drink.session.get_absolute_url())
    drink_url = event.drink.ShortUrl()

  beer_name = ''
  if event.drink.keg and event.drink.keg.type:
    beer_name = event.drink.keg.type.name
    
  drink_size = ''
  if event.drink:
    drink_size = '%.1foz' % event.drink.Volume().InOunces()

  kbvars = {
    'kb_name': event.site.settings.title,
    'name': name,
    'kb_url': base_url,
    'drink_url': drink_url,
    'session_url': session_url,
    'drink_size': drink_size,
    'beer_name': beer_name,
  }
  return kbvars

def do_checkin(event):
  drink = event.drink

  beerid = drink.keg.type.untappd_beer_id

  if beerid == None:
    logger.info('Untappd beer id is not specified for this beer.')
    return

  ounces = drink.Volume().InOunces()

  min_ounces = 2.0

  if ounces < min_ounces:
    logger.info('Drink too small to care about; no untappd checkin for you!')
    return

  if event.user:
    try:
      db_user = core_models.User.objects.get(username=event.user.username)
      profile = db_user.get_profile()
    except (core_models.User.DoesNotExist, core_models.UserProfile.DoesNotExist):
      logger.info('Profile for user %s does not exist' % drink.user)
      return
    try:
      untappd_link = models.UserUntappdLink.objects.get(user_profile=profile)

    except models.UserUntappdLink.DoesNotExist:
      # Untappd name unknown
      logger.info('User %s has not enabled untappd link.' % drink.user)
      return

  try:
    site_4sq_settings = foursquare_models.SiteFoursquareSettings.objects.get(site=event.site)
    #venue_id = site_4sq_settings.venue_id
    venue_id = ''
  except foursquare_models.SiteFoursquareSettings.DoesNotExist:
    venue_id = ''

  kbvars = _get_vars(event)

  shout = DEFAULT_CHECKIN_TEMPLATE % kbvars

  params = {
    'bid': beerid,
    'gmt_offset': settings.GMT_OFFSET,
    'shout': shout,
  }

  if venue_id:
    params['foursquare_id'] = venue_id

  params = urlencode(params)
  req = urllib2.Request("http://api.untappd.com/v3/checkin?key=" + settings.UNTAPPD_API_KEY, params)

  base64string = base64.encodestring('%s:%s' % (untappd_link.untappd_name, untappd_link.untappd_password_hash))[:-1]
  authheader =  "Basic %s" % base64string
  req.add_header("Authorization", authheader)

  response = urllib2.urlopen(req)

  response = response.read()
  logger.info('Response: %s' % str(response))



#!/usr/bin/env python
"""Kegbot kegerator controller software

This package contains Kegbot core controller and Django
frontend package.

Kegbot is a hardware and software system to record and monitor access to a beer
kegerator.  For more information and documentation, see http://kegbot.org/
"""

DOCLINES = __doc__.split('\n')

# Change this to True to include optional dependencies
USE_OPTIONAL = False

VERSION = '0.9.4'
SHORT_DESCRIPTION = DOCLINES[0]
LONG_DESCRIPTION = '\n'.join(DOCLINES[2:])
REQUIRED = [
  'kegbot-pyutils >= 0.1.4',
  'kegbot-api >= 0.1.2',

  'django >= 1.3',
  'django-autoslug',
  'django-imagekit >= 2.0',
  'django-registration',
  'django-socialregistration >= 0.5.4',
  'django_extensions',

  'MySQL-python',
  'South >= 0.7.3',
  'django-crispy-forms',
  'django-icanhaz',
  'django_nose',
  'facebook-sdk >= 0.3.0',
  'foursquare',
  'gunicorn >= 0.16.1',
  'pil',
  'poster', # needed by foursquare
  'protobuf >= 2.4.1',
  'pylcdui >= 0.5.5',
  'pysqlite>=2.0.3',
  'python-gflags >= 1.8',
  'pytz',
  'requests', # needed by oauth
  'tweepy',
]

OPTIONAL = [
  ### Optional modules.
  # This modules 'pip install'd manually.  If you add to this list, also
  # update pykeg.core.optional_modules with a new 'HAVE_<MODULE>' flag, as
  # well as pykeg.settings (if needed).

  ### django-debug-toolbar
  'django-debug-toolbar',

  ### Celery
  'Celery',
  'django-celery',
  'django-kombu',

  ### Raven
  'raven',

  ### Sentry
  'sentry',
]

DEPENDENCIES = REQUIRED
if USE_OPTIONAL:
  DEPENDENCIES += OPTIONAL


def setup_package():
  from distribute_setup import use_setuptools
  use_setuptools()
  from setuptools import setup, find_packages

  setup(
      name = 'kegbot',
      version = VERSION,
      description = SHORT_DESCRIPTION,
      long_description = LONG_DESCRIPTION,
      author = 'mike wakerly',
      author_email = 'opensource@hoho.com',
      url = 'http://kegbot.org/',
      packages = find_packages('src'),
      package_dir = {
        '' : 'src',
      },
      scripts = [
        'distribute_setup.py',
        'bin/kegbot-admin.py',
      ],
      install_requires = DEPENDENCIES,
      dependency_links = [
          'https://github.com/rem/python-protobuf/tarball/master#egg=protobuf-2.4.1',
      ],
      include_package_data = True,
      entry_points = {
        'console_scripts': ['instance=django.core.management:execute_manager'],
      },

  )

if __name__ == '__main__':
  setup_package()



# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Celery tasks for Twitter."""

from kegbot.util import util
from kegbot.util import kbjson

from pykeg.connections import common

from urllib import urlencode
import urllib2
import logging
import tweepy
from django.contrib.sites.models import Site

from celery.task import task

logger = common.get_logger(__name__)

from django.conf import settings

@task
def tweet_event(event):
  if common.is_stale(event.time):
    logger.info('Event is stale, ignoring: %s' % str(event))
    return
  if event.kind not in ('session_joined', 'drink_poured'):
    logger.info('Event is not tweetable: %s' % event.kind)
    return
  kbvars = _get_vars(event)
  do_user_tweet(event, kbvars)
  do_system_tweet(event, kbvars)

def add_hashtag(tweet):
  if len(tweet) <= (140 - len(util.HASHTAG) - 1):
    tweet = '%s %s' % (tweet, util.HASHTAG)
  return tweet

def _get_vars(event):
  base_url = 'http://%s/%s' % (Site.objects.get_current().domain, event.site.url())
  name = ''
  if event.user:
    name = event.user.username
  session_url = ''
  drink_url = ''

  if event.drink:
    session_url = '%s/%s' % (base_url, event.drink.session.get_absolute_url())
    drink_url = event.drink.ShortUrl()

  beer_name = ''
  if event.drink.keg and event.drink.keg.type:
    beer_name = event.drink.keg.type.name

  drink_size = ''
  if event.drink:
    drink_size = '%.1foz' % event.drink.Volume().InOunces()

  kbvars = {
    'kb_name': event.site.settings.title,
    'name': name,
    'kb_url': base_url,
    'drink_url': drink_url,
    'session_url': session_url,
    'drink_size': drink_size,
    'beer_name': beer_name,
  }
  return kbvars


def do_system_tweet(event, kbvars):
  """Sends a tweet using the system's account (if any)."""
  system_profile = util.get_site_profile(event.site)
  if not system_profile:
    logger.info('No system twitter profile, skipping system tweet')
    return

  user = event.user
  if user:
    user_profile = util.get_user_profile(user)
    if user_profile:
      name = '@%s' % (user_profile.settings.twitter_name,)
    elif not system_profile.settings.post_unlinked:
      logger.info('Tweets for unlinked users are disabled')
      return
    else:
      name = user.username
  elif not system_profile.settings.post_unauthenticated:
    logger.info('Tweets for anonymous pours are disabled.')
    return
  else:
    name = 'Someone'

  kbvars['name'] = name

  system_api = util.get_api(system_profile.oauth_token,
      system_profile.oauth_token_secret)

  tweet = None
  if event.kind == 'session_joined':
    tweet = util.DEFAULT_SYSTEM_SESSION_JOINED_TEMPLATE % kbvars
  elif event.kind == 'drink_poured':
    tweet = util.DEFAULT_SYSTEM_DRINK_POURED_TEMPLATE % kbvars

  if tweet:
    tweet = add_hashtag(tweet)
    logger.info('Sending system tweet: %s' % tweet)
    try:
      ret = system_api.update_status(tweet)
    except tweepy.TweepError, e:
      logger.error('Error posting tweet: %s' % str(e))

def do_user_tweet(event, kbvars):
  """Sends a tweet using the user's account (if any)."""
  user = event.user
  if not user:
    logger.info('No user for this event, no user tweet possible: %s' % str(event))
    return

  profile = util.get_user_profile(user)
  if not profile or not profile.settings.enabled:
    logger.info('User has disabled Twitter: %s' % user)
    return

  kind = event.kind
  tweet = None
  if kind == 'drink_poured':
    if not profile.settings.post_drink_poured:
      logger.info('User has disabled drink pour tweets: %s' % event.user)
      return
    tweet = util.DEFAULT_USER_DRINK_POURED_TEMPLATE % kbvars
  elif kind == 'session_joined':
    if not profile.settings.post_session_joined:
      logger.info('User has disabled session-join tweets: %s' % event.user)
      return
    tweet = util.DEFAULT_USER_SESSION_JOINED_TEMPLATE % kbvars

  user_api = util.get_api(profile.access_token.oauth_token,
      profile.access_token.oauth_token_secret)

  if tweet:
    tweet = add_hashtag(tweet)
    logger.info('Sending user tweet: %s' % tweet)
    try:
      ret = user_api.update_status(tweet)
    except tweepy.TweepError, e:
      logger.error('Error posting tweet: %s' % str(e))



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django import forms

from pykeg.core import models

class DrinkPostForm(forms.Form):
  """Form to handle posts to /tap/<tap_id>/"""
  ticks = forms.IntegerField()
  volume_ml = forms.FloatField(required=False)
  username = forms.CharField(required=False)
  pour_time = forms.IntegerField(required=False)
  now = forms.IntegerField(required=False)
  duration = forms.IntegerField(required=False)
  auth_token = forms.CharField(required=False)
  spilled = forms.BooleanField(required=False)
  shout = forms.CharField(required=False)

class CancelDrinkForm(forms.Form):
  """Form to handled posts to /cancel-drink/"""
  id = forms.IntegerField()
  spilled = forms.BooleanField(required=False)

class ThermoPostForm(forms.Form):
  """Handles posting new temperature sensor readings."""
  temp_c = forms.FloatField()
  when = forms.IntegerField(required=False)
  now = forms.IntegerField(required=False)

class CreateKegTapForm(forms.ModelForm):
  class Meta:
    model = models.KegTap
    fields = ('name', 'meter_name', 'relay_name', 'ml_per_tick', 'description')

class PhotoForm(forms.Form):
  photo = forms.ImageField()
  time = forms.CharField(required=False)

class DebugLogForm(forms.Form):
  message = forms.CharField()
  client_name = forms.CharField(required=False)

class RegisterForm(forms.Form):
  username = forms.CharField(max_length=32)
  email = forms.EmailField()
  password = forms.CharField()
  photo = forms.ImageField(required=False)

class AssignTokenForm(forms.Form):
  username = forms.CharField()




# Copyright 2011 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from pykeg.core import models
from pykeg.web.api.views import check_api_key

from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template.response import SimpleTemplateResponse
from django.template import RequestContext

ALLOWED_PATHS = (
    '/api/login/',
    '/accounts/login/',
    '/admin/',
    '/media/',
)

def _path_allowed(path, kbsite):
  if kbsite:
    path = path.lstrip(kbsite.url())
  for p in ALLOWED_PATHS:
    if path.startswith(p):
      return True
  return False


class KegbotSiteMiddleware:
  def process_view(self, request, view_func, view_args, view_kwargs):
    """Removes kbsite_name from kwargs if present, and attaches the
    corresponding KegbotSite instance to the request as the "kbsite" attribute.

    If kbsite_name is None, the default site is selected.
    """
    kbsite_name = view_kwargs.pop('kbsite_name', None)
    if not kbsite_name:
      kbsite_name = 'default'
    try:
      request.kbsite = models.KegbotSite.objects.get(name=kbsite_name)
    except models.KegbotSite.DoesNotExist:
      pass
    return None

class SiteActiveMiddleware:
  """Middleware which throws 503s when KegbotSite.is_active is false."""
  def process_view(self, request, view_func, view_args, view_kwargs):
    if not hasattr(request, 'kbsite'):
      return None
    kbsite = request.kbsite

    # We have a KegbotSite, and that site is active: nothing to do.
    if kbsite.is_active:
      return None

    # If the request is for a whitelisted path, allow it.
    if _path_allowed(request.path, kbsite):
      return None

    # Allow staff/superusers access if inactive.
    if request.user.is_staff or request.user.is_superuser:
      return None

    return HttpResponse('Site temporarily unavailable', status=503)

class PrivacyMiddleware:
  def process_view(self, request, view_func, view_args, view_kwargs):
    if not hasattr(request, 'kbsite'):
      return None
    elif _path_allowed(request.path, request.kbsite):
      return None
    elif getattr(request, 'kb_api_authenticated', False):
      # This is an auth-required kb api view; no need to check privacy since API
      # keys are given staff-level access.
      return None

    privacy = request.kbsite.settings.privacy
    if privacy == 'public':
      return None

    # If non-public, apply the API key check.
    if view_func.__module__.startswith('pykeg.web.api'):
      check_api_key(request)

    if privacy == 'staff' and not request.user.is_staff:
      return SimpleTemplateResponse('kegweb/staff_only.html',
          context=RequestContext(request), status=503)
    elif privacy == 'members':
      if not request.user.is_authenticated or not request.user.is_active:
        return SimpleTemplateResponse('kegweb/members_only.html',
            context=RequestContext(request), status=503)
      return None

    return HttpResponse('Server misconfigured, unknown privacy setting:%s' % privacy, status=503)

